@@ -92,16 +92,16 @@ KCRASH_EXPORT bool loadedByKdeinit = false;
 void setApplicationFilePath(const QString &filePath);
 }
 
-static KCrash::HandlerType s_emergencySaveFunction = 0;
-static KCrash::HandlerType s_crashHandler = 0;
-static char *s_appName = 0;
-static char *s_autoRestartCommand = 0;
-static char *s_appPath = 0;
+static KCrash::HandlerType s_emergencySaveFunction = nullptr;
+static KCrash::HandlerType s_crashHandler = nullptr;
+static char *s_appName = nullptr;
+static char *s_autoRestartCommand = nullptr;
+static char *s_appPath = nullptr;
 static int s_autoRestartArgc = 0;
-static char **s_autoRestartCommandLine = 0;
-static char *s_drkonqiPath = 0;
-static char *s_kdeinit_socket_file = 0;
-static KCrash::CrashFlags s_flags = 0;
+static char **s_autoRestartCommandLine = nullptr;
+static char *s_drkonqiPath = nullptr;
+static char *s_kdeinit_socket_file = nullptr;
+static KCrash::CrashFlags s_flags = KCrash::CrashFlags();
 static int s_launchDrKonqi = -1; // -1=initial value 0=disabled 1=enabled
 Q_GLOBAL_STATIC(KCrash::CoreConfig, s_coreConfig)
 
@@ -131,6 +131,8 @@ void KCrash::initialize()
         && !qEnvironmentVariableIsSet("KCRASH_AUTO_RESTARTED")) {
         // enable drkonqi
         KCrash::setDrKonqiEnabled(true);
+    } else {
+        qCDebug(LOG_KCRASH) << "KCrash disabled through environment.";
     }
 
     KCrash::setApplicationFilePath(QCoreApplication::applicationFilePath());
@@ -219,7 +221,7 @@ void KCrash::setApplicationFilePath(const QString &filePath)
     for (int i = 0; i < args.count(); ++i) {
         s_autoRestartCommandLine[i] = qstrdup(QFile::encodeName(args.at(i)).constData());
     }
-    s_autoRestartCommandLine[args.count()] = 0;
+    s_autoRestartCommandLine[args.count()] = nullptr;
 }
 
 void KCrash::setDrKonqiEnabled(bool enabled)
@@ -256,42 +258,42 @@ bool KCrash::isDrKonqiEnabled()
     return s_launchDrKonqi == 1;
 }
 
-// The following function copy&pasted from kinit/wrapper.cpp :
-static char *getDisplay()
+// The following functions copy&pasted from kinit/wrapper.cpp :
+// (which copied it from kdeinit/kinit.cpp)
+// Can't use QGuiApplication::platformName() here, there is no app instance.
+#if HAVE_X11 || HAVE_XCB
+static const char* displayEnvVarName_c()
 {
-    const char *display;
-    char *result;
-    char *screen;
-    char *colon;
-    char *i;
-#if defined(NO_DISPLAY) || defined(Q_OS_WIN)
-    display = "NODISPLAY";
-#else
-    display = getenv("DISPLAY");
+    return "DISPLAY";
+}
 #endif
-    if (!display || !*display) {
-        display = ":0";
+
+// adapted from kdeinit/kinit.cpp
+// WARNING, if you change the socket name, adjust kinit.cpp too
+static const QString generate_socket_file_name()
+{
+
+#if HAVE_X11 || HAVE_XCB
+    QByteArray display = qgetenv(displayEnvVarName_c());
+    if (display.isEmpty()) {
+        fprintf(stderr, "Error: could not determine $%s.\n", displayEnvVarName_c());
+        return QString();
     }
-    result = (char *)malloc(strlen(display) + 1);
-    if (result == NULL) {
-        return NULL;
+    int i;
+    if ((i = display.lastIndexOf('.')) > display.lastIndexOf(':') && i >= 0) {
+        display.truncate(i);
     }
 
-    strcpy(result, display);
-    screen = strrchr(result, '.');
-    colon = strrchr(result, ':');
-    if (screen && (screen > colon)) {
-        *screen = '\0';
-    }
-    while ((i = strchr(result, ':'))) {
-        *i = '_';
-    }
+    display.replace(':', '_');
 #ifdef __APPLE__
-    while ((i = strchr(result, '/'))) {
-        *i = '_';
-    }
+    display.replace('/', '_');
 #endif
-    return result;
+#else
+    // not using a DISPLAY variable; use an empty string instead
+    QByteArray display = "";
+#endif
+    const QString socketFileName = QString::fromLatin1("kdeinit5_%1").arg(QLatin1String(display));
+    return socketFileName;
 }
 
 void
@@ -299,9 +301,7 @@ KCrash::setCrashHandler(HandlerType handler)
 {
     if (!s_kdeinit_socket_file) {
         // Prepare this now to avoid mallocs in the crash handler.
-        char *display = getDisplay();
-        const QString socketFileName = QStringLiteral("kdeinit5_%1").arg(QLatin1String(display));
-        free(display);
+        const QString socketFileName = generate_socket_file_name();
         QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) +
                                 QLatin1Char('/') + socketFileName);
         s_kdeinit_socket_file = qstrdup(socketName.constData());
@@ -345,7 +345,7 @@ KCrash::setCrashHandler(HandlerType handler)
     sigaddset(&mask, SIGABRT);
 #endif
 
-    sigprocmask(SIG_UNBLOCK, &mask, 0);
+    sigprocmask(SIG_UNBLOCK, &mask, nullptr);
 #endif
 
     s_crashHandler = handler;
@@ -357,7 +357,7 @@ KCrash::crashHandler()
     return s_crashHandler;
 }
 
-#ifndef Q_OS_WIN
+#if !defined(Q_OS_WIN) && !defined(Q_OS_OSX)
 static void
 closeAllFDs()
 {
@@ -398,7 +398,7 @@ KCrash::defaultCrashHandler(int sig)
         crashRecursionCounter++;
     }
 
-#if !defined(Q_OS_WIN)
+#if !defined(Q_OS_WIN) && !defined(Q_OS_OSX)
     if (!(s_flags & KeepFDs)) {
         closeAllFDs();
     }
@@ -427,7 +427,7 @@ KCrash::defaultCrashHandler(int sig)
 #endif
 
         if (s_launchDrKonqi != 1) {
-            setCrashHandler(0);
+            setCrashHandler(nullptr);
 #if !defined(Q_OS_WIN)
             raise(sig); // dump core, or whatever is the default action for this signal.
 #endif
@@ -523,7 +523,7 @@ KCrash::defaultCrashHandler(int sig)
 #endif
 
         // NULL terminated list
-        argv[i] = NULL;
+        argv[i] = nullptr;
 
         startProcess(i, argv, true);
     }
@@ -534,7 +534,7 @@ KCrash::defaultCrashHandler(int sig)
 
     if (s_coreConfig->isProcess()) {
         fprintf(stderr, "Re-raising signal for core dump handling.\n");
-        KCrash::setCrashHandler(0);
+        KCrash::setCrashHandler(nullptr);
         raise(sig);
         // not getting here
     }
@@ -616,6 +616,7 @@ void KCrash::startProcess(int argc, const char *argv[], bool waitAndExit)
 {
     bool startDirectly = true;
 
+#ifndef Q_OS_OSX
     // First try to start the app via kdeinit, if the AlwaysDirectly flag hasn't been specified.
     // This is done because it is dangerous to use fork() in the crash handler
     // (there can be functions registered to be performed before fork(), for example handling
@@ -623,6 +624,7 @@ void KCrash::startProcess(int argc, const char *argv[], bool waitAndExit)
     if (!(s_flags & AlwaysDirectly)) {
         startDirectly = !startProcessInternal(argc, argv, waitAndExit, false);
     }
+#endif
 
     // If we can't reach kdeinit, we can still at least try to fork()
     if (startDirectly) {
@@ -644,7 +646,7 @@ static bool startProcessInternal(int argc, const char *argv[], bool waitAndExit,
         // when launching drkonqi. Note that drkonqi will stop this process in the meantime.
         if (directly) {
             //if the process was started directly, use waitpid(), as it's a child...
-            while (waitpid(-1, NULL, 0) != pid) {}
+            while (waitpid(-1, nullptr, 0) != pid) {}
         } else {
 #ifdef Q_OS_LINUX
             // Declare the process that will be debugging the crashed KDE app (#245529)
@@ -734,11 +736,13 @@ static pid_t startDirectly(const char *argv[])
         fprintf(stderr, "KCrash failed to fork(), errno = %d\n", errno);
         return 0;
     case 0:
-        setgroups(0, 0); // Remove any extraneous groups
+        setgroups(0, nullptr); // Remove any extraneous groups
         if (setgid(getgid()) < 0 || setuid(getuid()) < 0) {
             _exit(253);    // This cannot happen. Theoretically.
         }
+#ifndef Q_OS_OSX
         closeAllFDs(); // We are in the child now. Close FDs unconditionally.
+#endif
         setenv("KCRASH_AUTO_RESTARTED", "1", 1);
         execvp(argv[0], const_cast< char ** >(argv));
         fprintf(stderr, "KCrash failed to exec(), errno = %d\n", errno);
